@@ -9,6 +9,7 @@ from autoprotocol import Protocol
 import requests
 from .version import __version__
 import platform
+import inspect
 
 
 class Connection(object):
@@ -65,7 +66,7 @@ class Connection(object):
     """
     def __init__(self, email=None, token=None, organization_id=False,
                  api_root="https://secure.transcriptic.com", organization=False,
-                 cookie=False, verbose=False, use_environ=True):
+                 cookie=False, verbose=False, use_environ=True, analytics=True, user_id="default"):
         if email is None and use_environ:
             email = os.environ['USER_EMAIL']
             token = os.environ['USER_TOKEN']
@@ -75,6 +76,8 @@ class Connection(object):
         self.token = token
         self.organization_id = organization_id or organization
         self.verbose = verbose
+        self.analytics = analytics
+        self.user_id = user_id
         self.headers = {
             "X-User-Email": email,
             "X-User-Token": token,
@@ -97,6 +100,10 @@ class Connection(object):
         """Loads connection from file"""
         with open(expanduser(path), 'r') as f:
             cfg = json.loads(f.read())
+            expected_keys = ['email', 'token', 'organization_id', 'api_root', 'analytics', "user_id"]
+
+            def key_not_found(): raise OSError("Key not found")
+            [key_not_found() for key in expected_keys if key not in cfg.keys()]
             return Connection(**cfg)
 
     def save(self, path):
@@ -107,6 +114,8 @@ class Connection(object):
                 'token': self.token,
                 'organization_id': self.organization_id,
                 'api_root': self.api_root,
+                'analytics': self.analytics,
+                'user_id': self.user_id
             }, indent=2))
 
     def update_environment(self, **kwargs):
@@ -251,10 +260,10 @@ class Connection(object):
         route = self.get_route('query_kits', query=query)
         return self.get(route)
 
-    def monitoring_data(self, data_type, project_id=None, run_id=None, instruction_id=None):
+    def monitoring_data(self, data_type, instruction_id=None, grouping=None, start_time=None, end_time=None):
         """Get monitoring_data"""
-        route = self.get_route('monitoring_data', project_id=project_id, run_id=run_id,
-                               instruction_id=instruction_id, data_type=data_type)
+        route = self.get_route('monitoring_data', data_type=data_type, instruction_id=instruction_id,
+                               grouping=grouping, start_time=start_time, end_time=end_time)
         return self.get(route)
 
     def raw_image_data(self, data_id=None):
@@ -315,12 +324,12 @@ class Connection(object):
         route = self.get_route('dataset', data_id=data_id, key=key)
         return self.get(route)
 
-    def datasets(self, project_id=None, run_id=None):
+    def datasets(self, project_id=None, run_id=None, timeout=30.0):
         """Get datasets belonging to run"""
         route = self.get_route('datasets', project_id=project_id, run_id=run_id)
         return self.get(route, status_response={
             '404': lambda resp: Exception("[404] No run found for ID " + id)
-        })
+        }, timeout=timeout)
 
     def get_zip(self, data_id, file_path=None):
         """
@@ -370,7 +379,9 @@ class Connection(object):
     def get_route(self, method, **kwargs):
         """Helper function to automatically match and supply required arguments"""
         route_method = getattr(routes, method)
-        route_method_args = route_method.__code__.co_varnames
+        route_method_args, _, _, route_defaults = inspect.getargspec(route_method)
+        if route_defaults:
+            route_method_args = route_method_args[:-len(route_defaults)]
         # Update loaded argument dictionary with additional arguments which are not None
         arg_dict = dict(self.env_args, **{k: v for k, v in list(kwargs.items()) if v is not None})
         input_args = []
@@ -440,6 +451,13 @@ class Connection(object):
         else:
             return return_val(response)
 
+    def _post_analytics(self, client_id=None, event_action=None, event_category="cli"):
+        route = "https://www.google-analytics.com/collect"
+        if not client_id:
+            client_id = self.user_id
+        packet = 'v=1&tid=UA-28937242-7&cid={}&t=event&ea={}&ec={}'.format(client_id, event_action, event_category)
+        requests.post(route, packet)
+
 
 class AnalysisException(Exception):
     def __init__(self, message):
